#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#include <Adafruit_INA219.h>
 
Adafruit_INA219 ina219;
LiquidCrystal_I2C lcd (0x27,16,2); // I2C address should be correct!!

// SD card library
#include <SPI.h>
#include <SD.h>
// SD card related definition
#define SD_CS   53 // SD card select pin
// CS is assigned as 53
// MOSI is 51
// MISO is 50
// SCK is 52
File myFile;
String file_name;

// DS3231 RTC
#include "RTClib.h"
RTC_DS3231 rtc;  // RTC 구조체 변수 선언
// 요일 이름 저장 배열
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
String dates = "";
String times = "";

void setup() {
  Serial.begin(9600);
  Serial.print("Program started...");
  lcd.init();
  lcd.setBacklight(HIGH);

  if (!ina219.begin()) {
    Serial.println("Failed to find INA219 chip");
    while (1) { delay(10); }
  }

  if(!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  // SD card is pretty straightforward, a single call...
  if (!SD.begin(SD_CS)) {
    Serial.println("SD initialization failed!");
    for(;;); // Fatal error, do not continue
  }

  get_time_stamp();
  myFile = SD.open(file_name, FILE_WRITE);
}

void loop() {
  float shuntvoltage = 0;
  float busvoltage = 0;
  float current_mA = 0;
  float loadvoltage = 0;
  float power_mW = 0;
 
  shuntvoltage = ina219.getShuntVoltage_mV();
  busvoltage = ina219.getBusVoltage_V();
  current_mA = ina219.getCurrent_mA();
  power_mW = ina219.getPower_mW();
  loadvoltage = busvoltage + (shuntvoltage / 1000);
  
  Serial.println("Bus Voltage:   "); 
  Serial.println(busvoltage); 
  Serial.println(" V");
  Serial.println("Shunt Voltage: "); 
  Serial.println(shuntvoltage); 
  Serial.println(" mV");
  Serial.println("Load Voltage:  "); 
  Serial.print(loadvoltage); 
  Serial.println(" V");
  Serial.print("Current:       "); 
  Serial.print(current_mA); Serial.println(" mA");
  Serial.print("Power:         "); Serial.print(power_mW); Serial.println(" mW");
  Serial.println("");

  lcd.setCursor(0, 0);
  lcd.print(" ");
  lcd.print(current_mA);
  lcd.print("mA");

  lcd.print(" ");
  lcd.print(loadvoltage);
  lcd.print("V");

  get_time_stamp();
  
  lcd.setCursor(0, 1);
  lcd.print(" ");
  lcd.print(power_mW);
  lcd.print("mW");
  lcd.print(" ");
  lcd.print(times);
  Serial.println(dates);

  if (!SD.exists(&file_name[0])) {
    myFile = SD.open(file_name, FILE_WRITE);
  }
  myFile.println(dates+times+","+(String) current_mA+"mA,"+(String) loadvoltage+"V,"+(String) power_mW+"mW");
  myFile.close();
  myFile = SD.open(file_name, FILE_WRITE); 
  
  delay(5000);  
  lcd.clear();
}

String year_data = "";
String month_data = "";
String day_data = "";

void get_time_stamp() {
  DateTime now = rtc.now();   // 현재 날짜, 시간을 모듈에서 가져옵니다.
    
  // Serial.print(now.year(), DEC);  // 년도 표시
  // Serial.print('/');
  // Serial.print(now.month(), DEC); // 월 표시
  // Serial.print('/');
  // Serial.print(now.day(), DEC);   // 일 표시
  year_data = (String) now.year();
  month_data = (String) now.month();
  day_data = (String) now.day();
  dates = year_data+"/"+month_data+"/"+day_data+"-";
  file_name = year_data + month_data + day_data + ".txt";
   
  Serial.print(" (");
  Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);  // 요일 표시
  Serial.print(") ");
 
  Serial.print(now.hour(), DEC);  // 시간 표시
  Serial.print(':');
  Serial.print(now.minute(), DEC);// 분 표시
  Serial.print(':');
  Serial.print(now.second(), DEC);// 초 표시
  Serial.println();  
  times = (String) now.hour()+":"+(String) now.minute()+":"+(String) now.second();
}
